@@ -14,7 +14,7 @@ use Carp;
 use Data::Table::Text qw(:all);
 use Digest::SHA qw(sha256_hex);
 use Encode;
-our $VERSION = 2017.724;
+our $VERSION = 2017.725;
 
 #1 Constructor                                                                  # Construct a new linter
 
@@ -31,7 +31,7 @@ genLValueScalarMethods(qw(docType));
 genLValueScalarMethods(qw(dtds));                                               # Optional directory containing the DTDs used to validate the xml
 genLValueScalarMethods(qw(errors));                                             # Number of lint errors detected by xmllint
 genLValueScalarMethods(qw(header));                                             # The first line: the xml header extracted from L<source|/source>
-genLValueScalarMethods(qw(labels));                                             # Optional parse tree to supply L<labels|Data::Edit::Xml/Labels> for the current L<file|/file>
+genLValueScalarMethods(qw(labels));                                             # Optional parse tree to supply L<labels|Data::Edit::Xml/Labels> for the current L<source|/source> as the labels are present in the parse tree not in the string representing the parse tree
 genLValueScalarMethods(qw(linted));                                             # Date the lint was performed by L<lint|/lint>
 genLValueScalarMethods(qw(idDefs));                                             # {id} = count - the number of times this id is defined in the xml contained in this L<file|/file>
 genLValueScalarMethods(qw(labelDefs));                                          # {label or id} = id - the id of the node containing a L<label|Data::Edit::Xml/Labels> defined on the xml
@@ -90,7 +90,9 @@ sub lint($@)
   my $T = "<!--linted: ".dateStamp." -->\n";                                    # Time stamp marks the start of the added comments
 
   my $L = sub                                                                   # Process any labels in the parse tree
-   {my $s = ''; $lint->labels->by(sub                                           # Search the supplied parse tree for any id or label definitions
+   {return '' unless $lint->labels;
+    my $s = '';
+    $lint->labels->by(sub                                                       # Search the supplied parse tree for any id or label definitions
      {my ($o) = @_;
 
       if (my $i = $o->id)                                                       # Id for this node but no labels
@@ -174,17 +176,25 @@ sub wait()
  {waitpid(pop @pids, 0) while @pids;                                            # Wait until sub processes have completed
  } # wait
 
-sub clear($@)                                                                   # Clear the results of a prior run
- {my ($outputDirectory, @fileExtensions) = @_;                                  # Directory to clear, extensions of files to remove
-  for my $dir($outputDirectory)                                                 # Directory
-   {for my $ext(@fileExtensions)                                                # Extensions
-     {unlink $_ for fileList(filePathExt($dir, qq(*), $ext));
+sub searchDirectoryTreeForMatchingFiles($@)                                     # Search a directory tree for files that match the specified extensions
+ {my ($folder, @extensions) = @_;                                               # Directory to start search in, extensions of files to find
+  my @f;                                                                        # Files
+  for my $dir($folder)                                                          # Directory
+   {for my $ext(@extensions)                                                    # Extensions
+     {push @f, fileList(filePathExt($dir, qq(*), $ext));                        # Record files that match
      }
    }
+  @f
+ } # fileList
+
+sub clear($@)                                                                   # Clear the results of a prior run
+ {my ($outputDirectory, @fileExtensions) = @_;                                  # Directory to clear, extensions of files to remove
+  unlink for &searchDirectoryTreeForMatchingFiles(@_);                          # Unlink the matching files
  } # clear
 
-sub relint($$@)                                                                 # Locate all the L<labels or id|Data::Edit::Xml/Labels> in the specified L<files|/file>, analyze the map of labels and ids with B<analysisSub> parse each L<file|/file>, process each parse with B<processSub>, then L<lint/lint> the reprocessed xml back to the original L<file|/file> - this allows you to reprocess the contents of each L<file|/file> with knowledge of where L<labels or id|Data::Edit::Xml/Labels> are located in the other L<files|/file> associated with a L<project|/project>. The B<analysisSub>(linkmap = {project}{labels or id>}=[file, id]) should return true if the processing of each file is to be performed subsequently. The B<processSub>(parse tree representation of a file, id and label mapping, reloaded linter) should return true if a L<lint|/lint> is required to save the results after each L<file|/file> has been processed else false, L<files|/file> to reprocess
- {my ($analysisSub, $processSub, @files) = @_;                                  # Analysis 𝘀𝘂𝗯, Process 𝘀𝘂𝗯, files to process
+sub relint($$$@)                                                                # Locate all the L<labels or id|Data::Edit::Xml/Labels> in the specified L<files|/file>, analyze the map of labels and ids with B<analysisSub> parse each L<file|/file>, process each parse with B<processSub>, then L<lint/lint> the reprocessed xml back to the original L<file|/file> - this allows you to reprocess the contents of each L<file|/file> with knowledge of where L<labels or id|Data::Edit::Xml/Labels> are located in the other L<files|/file> associated with a L<project|/project>. The B<analysisSub>(linkmap = {project}{labels or id>}=[file, id]) should return true if the processing of each file is to be performed subsequently. The B<processSub>(parse tree representation of a file, id and label mapping, reloaded linter) should return true if a L<lint|/lint> is required to save the results after each L<file|/file> has been processed else false, L<files|/file> to reprocess
+ {my ($analysisSub, $processSub, $folder, @extensions) = @_;                    # Analysis 𝘀𝘂𝗯, Process 𝘀𝘂𝗯, folder containing files to process (recursively), extensions of files to process
+  my @files = searchDirectoryTreeForMatchingFiles($folder, @extensions);        # Search for files to relint
 
   my $links;                                                                    # {project}{label or id} = [file, label or id]
   for my $file(@files)                                                          # Reload each file to reprocess
@@ -202,10 +212,12 @@ sub relint($$@)
 
   if ($analysisSub->($links))                                                   # Analyze links
    {for my $file(@files)                                                        # Reload, reparse, process, lint each file
-     {my $l = Data::Edit::Xml::Lint::read($file);                               # Reconstructed linter
-      my $x = Data::Edit::Xml::new($l->source);                                 # Reparse source
+     {my $lint = Data::Edit::Xml::Lint::read($file);                            # Reconstructed linter
+      my $x = eval{Data::Edit::Xml::new($lint->source)};                        # Reparse source trapping errors
+      $@ and warn "$@\nFailed to parse file:\n$file";                           # Xml file failed to parse
+      next if $@ or !$x;
 
-      if (my $links = $l->labelDefs)                                            # Reload labels
+      if (my $links = $lint->labelDefs)                                         # Reload labels
        {my $r;                                                                  # {primary id}[label]
         for my $source(sort keys %$links)                                       # Construct primary id to labels
          {my $target = $links->{$source};
@@ -224,8 +236,9 @@ sub relint($$@)
          });
        }
 
-      if ($processSub->($x, $links->{$l->project}, $l))                         # Call user method to process parse tree with labels in place and the location of all the labels and ids
-       {$l->source = join "\n", $l->header, $l->docType, $x->prettyString;      # Reconstruct source
+      if ($processSub->($x, $links->{$lint->project}, $lint))                   # Call user method to process parse tree with labels in place and the location of all the labels and ids
+       {my $l = $lint;                                                          # Shorten name
+        $l->source = join "\n", $l->header, $l->docType, $x->prettyString;      # Reconstruct source
         $l->labels = $x;                                                        # Associated parse tree so we can save the labels
         my %a = map {$_=>$l->{$_}} grep{!($l->can($_))} keys %$l;               # Reconstruct attributes as this items which do not have a method attached
         $l->lint(%a);                                                           # Lint reprocessed source
@@ -235,6 +248,14 @@ sub relint($$@)
   Data::Edit::Xml::Lint::wait();
  } # relint
 
+sub resolveUniqueLink($$)                                                       # Return the unique definition of the specified link in the link map or undef if no such definition exists
+ {my ($linkMap, $link) = @_;                                                    # Link map, label
+  my $l = $linkMap->{$link};                                                    # Attempt to resolve link
+  return undef unless $l;                                                       # No definition
+  return undef if @$l != 1;                                                     # Too many definitions
+  $l->[0][0]
+ } # resolveUniqueLink
+
 sub multipleLabelDefs($)                                                        # Return ([L<project|/project>; L<source label or id|Data::Edit::Xml/Labels>; targets count]*) of all L<labels or id|Data::Edit::Xml/Labels> that have multiple definitions
  {my ($labelDefs) = @_;                                                         # Label and Id definitions
   $labelDefs and ref($labelDefs) =~ /hash/is or                                 # Check definitions have been provided
@@ -416,9 +437,7 @@ if (1)
 
 # Tests and documentation
 
-sub test{eval join('', <Data::Edit::Xml::Lint::DATA>) or die $@}                ## Test
-
-test unless caller;
+sub test{eval join('', <Data::Edit::Xml::Lint::DATA>) or die $@}test unless caller;
 
 #extractDocumentation() unless caller();                                        ## podDocumentation
 
@@ -549,7 +568,6 @@ originating L<file|/file> thus fixing the changes
 
 =back
 
-
 =head1 Description
 
 =head2 Constructor
@@ -597,7 +615,7 @@ The first line: the xml header extracted from L<source|/source>
 
 =head4 labels :lvalue
 
-Optional parse tree to supply L<labels|Data::Edit::Xml/Labels> for the current L<file|/file>
+Optional parse tree to supply L<labels|Data::Edit::Xml/Labels> for the current L<source|/source> as the labels are present in the parse tree not in the string representing the parse tree
 
 
 =head4 linted :lvalue
@@ -659,6 +677,14 @@ Reread a linted xml L<file|/file> and extract the L<attributes|/Attributes> asso
 Wait for all L<lints|/lint> to finish - this is a static method, call as Data::Edit::Xml::Lint::wait
 
 
+=head3 searchDirectoryTreeForMatchingFiles
+
+Search a directory tree for files that match the specified extensions
+
+     Parameter    Description
+  1  $folder      Directory to start search in
+  2  @extensions  Extensions of files to find
+
 =head3 clear
 
 Clear the results of a prior run
@@ -674,7 +700,16 @@ Locate all the L<labels or id|Data::Edit::Xml/Labels> in the specified L<files|/
      Parameter     Description
   1  $analysisSub  Analysis 𝘀𝘂𝗯
   2  $processSub   Process 𝘀𝘂𝗯
-  3  @files        Files to process
+  3  $folder       Folder containing files to process (recursively)
+  4  @extensions   Extensions of files to process
+
+=head3 resolveUniqueLink
+
+Return the unique definition of the specified link in the link map or undef if no such definition exists
+
+     Parameter  Description
+  1  $linkMap   Link map
+  2  $link      Label
 
 =head3 multipleLabelDefs
 
@@ -809,6 +844,10 @@ L<relint|/relint>
 
 L<report|/report>
 
+L<resolveUniqueLink|/resolveUniqueLink>
+
+L<searchDirectoryTreeForMatchingFiles|/searchDirectoryTreeForMatchingFiles>
+
 L<sha256|/sha256>
 
 L<singleLabelDefs|/singleLabelDefs>
@@ -853,7 +892,7 @@ under the same terms as Perl itself.
 __DATA__
 use warnings FATAL=>qw(all);
 use strict;
-use Test::More tests=>107;
+use Test::More tests=>140;
 use Data::Edit::Xml;
 
 #Test::More->builder->output("/dev/null");                                       # Show only errors during testing - but this must be commented out for production
@@ -968,32 +1007,32 @@ END
          }
        }
 
-      is_deeply &mld, [Data::Edit::Xml::Lint::multipleLabelDefs
-                       ($labelDefs)];
-
-
-      is_deeply &sld, [Data::Edit::Xml::Lint::singleLabelDefs
-       ($labelDefs)];
+      is_deeply &mld, [Data::Edit::Xml::Lint::multipleLabelDefs($labelDefs)];
+      is_deeply &sld, [Data::Edit::Xml::Lint::singleLabelDefs($labelDefs)];
 
       1
      },
     sub                                                                         # Reprocess sub
      {my ($x, $labels, $lint) = @_;
+      my $s = $x->string;
+
       $x->by(sub                                                                # Search the supplied parse tree for xrefs to update
        {my ($o) = @_;
         if ($o->at(qw(xref)))
          {my $h = $o->href =~ s(\A#)()gsr;
-          if (my $l = $labels->{$h})
-           {$o->href = join "#", @{$$l[0]};
+          if (my $f = resolveUniqueLink($labels, $h))
+           {$o->href = "$f#$h";
            }
           else
            {confess "No target for $h in ".$lint->file;
            }
          }
-       });
+        });
+       ok $x->string ne $s;                                                     # Confirm that xml has been changed
+       ok $s         !~ /xml#c/;
+       ok $x->string =~ /xml#c/;
       1
-     },
-    @files
+     }, $outDir, "xml"
    );
 
   if (1)                                                                        # Confirm xref edits
@@ -1011,8 +1050,17 @@ END
     ok $s[9] =~ m(<xref href="out/bbb4.xml#c4"/>);
    }
 
-#  Data::Edit::Xml::Lint::clear($outDir, "xml");                                 # Remove results of last run
-#  rmdir $outDir;
+  if (1)
+   {my @f = searchDirectoryTreeForMatchingFiles($outDir, "xml");                # Remove results of last run
+    is_deeply [sort {$a cmp $b} @files],
+              [sort {$a cmp $b} @f];
+   }
+
+  Data::Edit::Xml::Lint::clear($outDir, "xml");                                 # Remove results of last run
+  ok !scalar(searchDirectoryTreeForMatchingFiles($outDir, "xml"));              # Confirm removal
+
+  rmdir $outDir;                                                                # Remove test folder
+  ok !-d $outDir;                                                               # Confirm removal
  }
 
 sub sld()                                                                       ## Expected single definition of labels or ids
@@ -1031,9 +1079,7 @@ sub sld()
  ]}
 
 sub mld()                                                                       ## Expected multiple definition of labels or ids
-
- {[
-  [
+ {[[
     "aaa",
     "aaa",
     [
@@ -1083,9 +1129,9 @@ sub mld()
     "bbb",
     [
       ["out/bbb1.xml", "bbb"],
+      ["out/bbb10.xml", "bbb"],
       ["out/bbb4.xml", "bbb"],
       ["out/bbb7.xml", "bbb"],
-      ["out/bbb10.xml", "bbb"],
     ],
   ],
   [
@@ -1093,9 +1139,9 @@ sub mld()
     "conbody1",
     [
       ["out/bbb1.xml", "c1"],
+      ["out/bbb10.xml", "c10"],
       ["out/bbb4.xml", "c4"],
       ["out/bbb7.xml", "c7"],
-      ["out/bbb10.xml", "c10"],
     ],
   ],
   [
@@ -1103,9 +1149,9 @@ sub mld()
     "conbody2",
     [
       ["out/bbb1.xml", "c1"],
+      ["out/bbb10.xml", "c10"],
       ["out/bbb4.xml", "c4"],
       ["out/bbb7.xml", "c7"],
-      ["out/bbb10.xml", "c10"],
     ],
   ],
   [
@@ -1113,9 +1159,9 @@ sub mld()
     "concept1",
     [
       ["out/bbb1.xml", "bbb"],
+      ["out/bbb10.xml", "bbb"],
       ["out/bbb4.xml", "bbb"],
       ["out/bbb7.xml", "bbb"],
-      ["out/bbb10.xml", "bbb"],
     ],
   ],
   [
@@ -1123,9 +1169,9 @@ sub mld()
     "concept2",
     [
       ["out/bbb1.xml", "bbb"],
+      ["out/bbb10.xml", "bbb"],
       ["out/bbb4.xml", "bbb"],
       ["out/bbb7.xml", "bbb"],
-      ["out/bbb10.xml", "bbb"],
     ],
   ],
   [
@@ -1173,7 +1219,7 @@ sub mld()
       ["out/ccc8.xml", "ccc"],
     ],
   ],
- ]}
+]}
 
 sub expectedReport
 {{failingFiles     => [
@@ -1216,9 +1262,3 @@ sub expectedRead
   sha256     => "b00cdebf2e1837fa15140d25315e5558ed59eb735b5fad4bade23969babf9531",
   source     => "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE concept PUBLIC \"-//HPE//DTD HPE DITA Concept//EN\" \"concept.dtd\" []>\n<concept >\n <title>Project bbb</title>\n <conbody >\n   <p>Body of bbb \xF0\x9D\x9D\xB0</p>\n   <p>See: <xref href=\"#c4\"/></p>\n </conbody>\n</concept>",
 }}
-
-=pod
-
-Now do do something in the reprocess sub and write a duplicate id report
-
-=cut
